@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// (C) Copyright Ion Gaztanaga 2005-2009. Distributed under the Boost
+// (C) Copyright Ion Gaztanaga 2005-2011. Distributed under the Boost
 // Software License, Version 1.0. (See accompanying file
 // LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 //
@@ -19,8 +19,13 @@
 #include <boost/interprocess/detail/workaround.hpp>
 #include <boost/interprocess/creation_tags.hpp>
 #include <boost/interprocess/detail/managed_memory_impl.hpp>
-#include <boost/interprocess/detail/move.hpp>
+#include <boost/move/move.hpp>
 #include <boost/assert.hpp>
+//These includes needed to fulfill default template parameters of
+//predeclarations in interprocess_fwd.hpp
+#include <boost/interprocess/mem_algo/rbtree_best_fit.hpp>  
+#include <boost/interprocess/sync/mutex_family.hpp>
+#include <boost/interprocess/indexes/iset_index.hpp>
 
 //!\file
 //!Describes a named user memory allocation user class. 
@@ -38,15 +43,16 @@ template
          template<class IndexConfig> class IndexType
       >
 class basic_managed_external_buffer 
-   : public detail::basic_managed_memory_impl <CharType, AllocationAlgorithm, IndexType>
+   : public ipcdetail::basic_managed_memory_impl <CharType, AllocationAlgorithm, IndexType>
 {
    /// @cond
-   typedef detail::basic_managed_memory_impl 
+   typedef ipcdetail::basic_managed_memory_impl 
       <CharType, AllocationAlgorithm, IndexType>    base_t;
-   BOOST_INTERPROCESS_MOVABLE_BUT_NOT_COPYABLE(basic_managed_external_buffer)
+   BOOST_MOVABLE_BUT_NOT_COPYABLE(basic_managed_external_buffer)
    /// @endcond
    
    public:
+   typedef typename base_t::size_type              size_type;
 
    //!Default constructor. Does nothing.
    //!Useful in combination with move semantics
@@ -55,10 +61,10 @@ class basic_managed_external_buffer
 
    //!Creates and places the segment manager. This can throw
    basic_managed_external_buffer
-      (create_only_t, void *addr, std::size_t size)
+      (create_only_t, void *addr, size_type size)
    {
       //Check if alignment is correct
-      BOOST_ASSERT((0 == (((std::size_t)addr) & (AllocationAlgorithm::Alignment - std::size_t(1u)))));
+      BOOST_ASSERT((0 == (((std::size_t)addr) & (AllocationAlgorithm::Alignment - size_type(1u)))));
       if(!base_t::create_impl(addr, size)){
          throw interprocess_exception("Could not initialize buffer in basic_managed_external_buffer constructor");
       }
@@ -66,30 +72,30 @@ class basic_managed_external_buffer
 
    //!Creates and places the segment manager. This can throw
    basic_managed_external_buffer
-      (open_only_t, void *addr, std::size_t size)
+      (open_only_t, void *addr, size_type size)
    {
       //Check if alignment is correct
-      BOOST_ASSERT((0 == (((std::size_t)addr) & (AllocationAlgorithm::Alignment - std::size_t(1u)))));
+      BOOST_ASSERT((0 == (((std::size_t)addr) & (AllocationAlgorithm::Alignment - size_type(1u)))));
       if(!base_t::open_impl(addr, size)){
          throw interprocess_exception("Could not initialize buffer in basic_managed_external_buffer constructor");
       }
    }
 
    //!Moves the ownership of "moved"'s managed memory to *this. Does not throw
-   basic_managed_external_buffer(BOOST_INTERPROCESS_RV_REF(basic_managed_external_buffer) moved)
+   basic_managed_external_buffer(BOOST_RV_REF(basic_managed_external_buffer) moved)
    {
       this->swap(moved);
    }
 
    //!Moves the ownership of "moved"'s managed memory to *this. Does not throw
-   basic_managed_external_buffer &operator=(BOOST_INTERPROCESS_RV_REF(basic_managed_external_buffer) moved)
+   basic_managed_external_buffer &operator=(BOOST_RV_REF(basic_managed_external_buffer) moved)
    {
-      basic_managed_external_buffer tmp(boost::interprocess::move(moved));
+      basic_managed_external_buffer tmp(boost::move(moved));
       this->swap(tmp);
       return *this;
    }
 
-   void grow(std::size_t extra_bytes)
+   void grow(size_type extra_bytes)
    {  base_t::grow(extra_bytes);   }
 
    //!Swaps the ownership of the managed heap memories managed by *this and other.
