@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// (C) Copyright Ion Gaztanaga 2005-2009. Distributed under the Boost
+// (C) Copyright Ion Gaztanaga 2005-2011. Distributed under the Boost
 // Software License, Version 1.0. (See accompanying file
 // LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 //
@@ -18,6 +18,7 @@
 #include <boost/interprocess/detail/config_begin.hpp>
 #include <boost/interprocess/detail/workaround.hpp>
 #include <boost/interprocess/sync/scoped_lock.hpp>
+#include <boost/interprocess/detail/posix_time_types_wrk.hpp>
 #include <boost/interprocess/sync/interprocess_mutex.hpp>
 #include <boost/interprocess/sync/interprocess_condition.hpp>
 #include <climits>
@@ -65,7 +66,7 @@ class interprocess_upgradable_mutex
    bool try_lock();
 
    //!Effects: The calling thread tries to acquire exclusive ownership of the mutex
-   //!   waiting if necessary until no other thread has has exclusive, sharable or
+   //!   waiting if necessary until no other thread has exclusive, sharable or
    //!   upgradable ownership of the mutex or abs_time is reached. 
    //!Returns: If acquires exclusive ownership, returns true. Otherwise returns false. 
    //!Throws: interprocess_exception on error.
@@ -79,13 +80,13 @@ class interprocess_upgradable_mutex
    //Sharable locking
 
    //!Effects: The calling thread tries to obtain sharable ownership of the mutex,
-   //!   and if another thread has exclusive or upgradable ownership of the mutex,
+   //!   and if another thread has exclusive ownership of the mutex,
    //!   waits until it can obtain the ownership.
    //!Throws: interprocess_exception on error.
    void lock_sharable();
 
    //!Effects: The calling thread tries to acquire sharable ownership of the mutex
-   //!   without waiting. If no other thread has has exclusive or upgradable ownership
+   //!   without waiting. If no other thread has exclusive ownership
    //!   of the mutex this succeeds. 
    //!Returns: If it can acquire sharable ownership immediately returns true. If it
    //!   has to wait, returns false. 
@@ -93,7 +94,7 @@ class interprocess_upgradable_mutex
    bool try_lock_sharable();
 
    //!Effects: The calling thread tries to acquire sharable ownership of the mutex
-   //!   waiting if necessary until no other thread has has exclusive or upgradable
+   //!   waiting if necessary until no other thread has exclusive
    //!   ownership of the mutex or abs_time is reached. 
    //!Returns: If acquires sharable ownership, returns true. Otherwise returns false. 
    //!Throws: interprocess_exception on error.
@@ -113,7 +114,7 @@ class interprocess_upgradable_mutex
    void lock_upgradable();
 
    //!Effects: The calling thread tries to acquire upgradable ownership of the mutex
-   //!   without waiting. If no other thread has has exclusive or upgradable ownership
+   //!   without waiting. If no other thread has exclusive or upgradable ownership
    //!   of the mutex this succeeds. 
    //!Returns: If it can acquire upgradable ownership immediately returns true.
    //!   If it has to wait, returns false.
@@ -121,7 +122,7 @@ class interprocess_upgradable_mutex
    bool try_lock_upgradable();
 
    //!Effects: The calling thread tries to acquire upgradable ownership of the mutex
-   //!   waiting if necessary until no other thread has has exclusive or upgradable
+   //!   waiting if necessary until no other thread has exclusive or upgradable
    //!   ownership of the mutex or abs_time is reached.
    //!Returns: If acquires upgradable ownership, returns true. Otherwise returns false. 
    //!Throws: interprocess_exception on error.
