@@ -27,13 +27,14 @@
 #include <boost/intrusive/detail/tree_node.hpp>
 #include <boost/intrusive/detail/ebo_functor_holder.hpp>
 #include <boost/intrusive/detail/mpl.hpp>
-#include <boost/intrusive/detail/pointer_to_other.hpp>
+#include <boost/intrusive/pointer_traits.hpp>
 #include <boost/intrusive/detail/clear_on_destructor_base.hpp>
 #include <boost/intrusive/detail/function_detector.hpp>
+#include <boost/intrusive/detail/utilities.hpp>
 #include <boost/intrusive/options.hpp>
 #include <boost/intrusive/rbtree_algorithms.hpp>
 #include <boost/intrusive/link_mode.hpp>
-//iG pending #include <boost/pointer_cast.hpp>
+#include <boost/move/move.hpp>
 
 namespace boost {
 namespace intrusive {
@@ -97,18 +98,19 @@ class rbtree_impl
    /// @endcond
    typedef typename real_value_traits::pointer                       pointer;
    typedef typename real_value_traits::const_pointer                 const_pointer;
-   typedef typename std::iterator_traits<pointer>::value_type        value_type;
+
+   typedef typename pointer_traits<pointer>::element_type            value_type;
    typedef value_type                                                key_type;
-   typedef typename std::iterator_traits<pointer>::reference         reference;
-   typedef typename std::iterator_traits<const_pointer>::reference   const_reference;
-   typedef typename std::iterator_traits<pointer>::difference_type   difference_type;
+   typedef typename pointer_traits<pointer>::reference               reference;
+   typedef typename pointer_traits<const_pointer>::reference         const_reference;
+   typedef typename pointer_traits<const_pointer>::difference_type   difference_type;
    typedef typename Config::size_type                                size_type;
    typedef typename Config::compare                                  value_compare;
    typedef value_compare                                             key_compare;
    typedef tree_iterator<rbtree_impl, false>                         iterator;
    typedef tree_iterator<rbtree_impl, true>                          const_iterator;
-   typedef std::reverse_iterator<iterator>                           reverse_iterator;
-   typedef std::reverse_iterator<const_iterator>                     const_reverse_iterator;
+   typedef boost::intrusive::detail::reverse_iterator<iterator>      reverse_iterator;
+   typedef boost::intrusive::detail::reverse_iterator<const_iterator>const_reverse_iterator;
    typedef typename real_value_traits::node_traits                   node_traits;
    typedef typename node_traits::node                                node;
    typedef typename node_traits::node_ptr                            node_ptr;
@@ -122,8 +124,7 @@ class rbtree_impl
    typedef detail::size_holder<constant_time_size, size_type>        size_traits;
 
    //noncopyable
-   rbtree_impl (const rbtree_impl&);
-   rbtree_impl operator =(const rbtree_impl&);
+   BOOST_MOVABLE_BUT_NOT_COPYABLE(rbtree_impl)
 
    enum { safemode_or_autounlink  = 
             (int)real_value_traits::link_mode == (int)auto_unlink   ||
@@ -158,17 +159,20 @@ class rbtree_impl
    value_compare &priv_comp()
    {  return data_.node_plus_pred_.get();  }
 
-   const node &priv_header() const
-   {  return data_.node_plus_pred_.header_plus_size_.header_;  }
+   const value_traits &priv_value_traits() const
+   {  return data_;  }
 
-   node &priv_header()
-   {  return data_.node_plus_pred_.header_plus_size_.header_;  }
+   value_traits &priv_value_traits()
+   {  return data_;  }
 
-   static node_ptr uncast(const_node_ptr ptr)
-   {
-      return node_ptr(const_cast<node*>(detail::boost_intrusive_get_pointer(ptr)));
-//iG pending return node_ptr(boost::const_pointer_cast<node>(ptr));
-   }
+   node_ptr priv_header_ptr()
+   {  return pointer_traits<node_ptr>::pointer_to(data_.node_plus_pred_.header_plus_size_.header_);  }
+
+   const_node_ptr priv_header_ptr() const
+   {  return pointer_traits<const_node_ptr>::pointer_to(data_.node_plus_pred_.header_plus_size_.header_);  }
+
+   static node_ptr uncast(const const_node_ptr & ptr)
+   {  return pointer_traits<node_ptr>::const_cast_from(ptr);  }
 
    size_traits &priv_size_traits()
    {  return data_.node_plus_pred_.header_plus_size_;  }
@@ -193,10 +197,10 @@ class rbtree_impl
    {  return priv_comp(); }
 
    const node &prot_header_node() const
-   {  return priv_header(); }
+   {  return data_.node_plus_pred_.header_plus_size_.header_; }
 
    node &prot_header_node()
-   {  return priv_header(); }
+   {  return data_.node_plus_pred_.header_plus_size_.header_; }
 
    void prot_set_size(size_type s)
    {  this->priv_size_traits().set_size(s);  }
@@ -224,7 +228,7 @@ class rbtree_impl
               , const value_traits &v_traits = value_traits()) 
       :  data_(cmp, v_traits)
    {  
-      node_algorithms::init_header(&priv_header());  
+      node_algorithms::init_header(this->priv_header_ptr());  
       this->priv_size_traits().set_size(size_type(0));
    }
 
@@ -246,7 +250,7 @@ class rbtree_impl
               , const value_traits &v_traits = value_traits())
       : data_(cmp, v_traits)
    {
-      node_algorithms::init_header(&priv_header());
+      node_algorithms::init_header(this->priv_header_ptr());
       this->priv_size_traits().set_size(size_type(0));
       if(unique)
          this->insert_unique(b, e);
@@ -254,6 +258,21 @@ class rbtree_impl
          this->insert_equal(b, e);
    }
 
+   //! <b>Effects</b>: to-do
+   //!   
+   rbtree_impl(BOOST_RV_REF(rbtree_impl) x)
+      : data_(::boost::move(x.priv_comp()), ::boost::move(x.priv_value_traits()))
+   {
+      node_algorithms::init_header(this->priv_header_ptr());  
+      this->priv_size_traits().set_size(size_type(0));
+      this->swap(x);
+   }
+
+   //! <b>Effects</b>: to-do
+   //!   
+   rbtree_impl& operator=(BOOST_RV_REF(rbtree_impl) x) 
+   {  this->swap(x); return *this;  }
+
    //! <b>Effects</b>: Detaches all elements from this. The objects in the set 
    //!   are not deleted (i.e. no destructors are called), but the nodes according to 
    //!   the value_traits template parameter are reinitialized and thus can be reused. 
@@ -270,7 +289,7 @@ class rbtree_impl
    //! 
    //! <b>Throws</b>: Nothing.
    iterator begin()
-   {  return iterator (node_traits::get_left(node_ptr(&priv_header())), this);   }
+   {  return iterator (node_traits::get_left(this->priv_header_ptr()), this);   }
 
    //! <b>Effects</b>: Returns a const_iterator pointing to the beginning of the tree.
    //! 
@@ -286,7 +305,7 @@ class rbtree_impl
    //! 
    //! <b>Throws</b>: Nothing.
    const_iterator cbegin() const
-   {  return const_iterator (node_traits::get_left(const_node_ptr(&priv_header())), this);   }
+   {  return const_iterator (node_traits::get_left(this->priv_header_ptr()), this);   }
 
    //! <b>Effects</b>: Returns an iterator pointing to the end of the tree.
    //! 
@@ -294,7 +313,7 @@ class rbtree_impl
    //! 
    //! <b>Throws</b>: Nothing.
    iterator end()
-   {  return iterator (node_ptr(&priv_header()), this);  }
+   {  return iterator (this->priv_header_ptr(), this);  }
 
    //! <b>Effects</b>: Returns a const_iterator pointing to the end of the tree.
    //!
@@ -310,7 +329,7 @@ class rbtree_impl
    //! 
    //! <b>Throws</b>: Nothing.
    const_iterator cend() const
-   {  return const_iterator (uncast(const_node_ptr(&priv_header())), this);  }
+   {  return const_iterator (uncast(this->priv_header_ptr()), this);  }
 
    //! <b>Effects</b>: Returns a reverse_iterator pointing to the beginning of the
    //!    reversed tree.
@@ -424,7 +443,7 @@ class rbtree_impl
    //! 
    //! <b>Throws</b>: Nothing.
    bool empty() const
-   {  return node_algorithms::unique(const_node_ptr(&priv_header()));   }
+   {  return node_algorithms::unique(this->priv_header_ptr());   }
 
    //! <b>Effects</b>: Returns the number of elements stored in the tree.
    //! 
@@ -437,7 +456,7 @@ class rbtree_impl
       if(constant_time_size)
          return this->priv_size_traits().get_size();
       else{
-         return (size_type)node_algorithms::size(const_node_ptr(&priv_header()));
+         return (size_type)node_algorithms::size(this->priv_header_ptr());
       }
    }
 
@@ -452,7 +471,7 @@ class rbtree_impl
       using std::swap;
       swap(priv_comp(), priv_comp());
       //These can't throw
-      node_algorithms::swap_tree(node_ptr(&priv_header()), node_ptr(&other.priv_header()));
+      node_algorithms::swap_tree(this->priv_header_ptr(), node_ptr(other.priv_header_ptr()));
       if(constant_time_size){
          size_type backup = this->priv_size_traits().get_size();
          this->priv_size_traits().set_size(other.priv_size_traits().get_size());
@@ -478,9 +497,10 @@ class rbtree_impl
       node_ptr to_insert(get_real_value_traits().to_node_ptr(value));
       if(safemode_or_autounlink)
          BOOST_INTRUSIVE_SAFE_HOOK_DEFAULT_ASSERT(node_algorithms::unique(to_insert));
+      iterator ret(node_algorithms::insert_equal_upper_bound
+         (this->priv_header_ptr(), to_insert, key_node_comp), this);
       this->priv_size_traits().increment();
-      return iterator(node_algorithms::insert_equal_upper_bound
-         (node_ptr(&priv_header()), to_insert, key_node_comp), this);
+      return ret;
    }
 
    //! <b>Requires</b>: value must be an lvalue, and "hint" must be
@@ -504,9 +524,10 @@ class rbtree_impl
       node_ptr to_insert(get_real_value_traits().to_node_ptr(value));
       if(safemode_or_autounlink)
          BOOST_INTRUSIVE_SAFE_HOOK_DEFAULT_ASSERT(node_algorithms::unique(to_insert));
+      iterator ret(node_algorithms::insert_equal
+         (this->priv_header_ptr(), hint.pointed_node(), to_insert, key_node_comp), this);
       this->priv_size_traits().increment();
-      return iterator(node_algorithms::insert_equal
-         (node_ptr(&priv_header()), hint.pointed_node(), to_insert, key_node_comp), this);
+      return ret;
    }
 
    //! <b>Requires</b>: Dereferencing iterator must yield an lvalue 
@@ -640,7 +661,7 @@ class rbtree_impl
          comp(key_value_comp, this);
       std::pair<node_ptr, bool> ret = 
          (node_algorithms::insert_unique_check
-            (node_ptr(&priv_header()), key, comp, commit_data));
+            (this->priv_header_ptr(), key, comp, commit_data));
       return std::pair<iterator, bool>(iterator(ret.first, this), ret.second);
    }
 
@@ -685,7 +706,7 @@ class rbtree_impl
          comp(key_value_comp, this);
       std::pair<node_ptr, bool> ret = 
          (node_algorithms::insert_unique_check
-            (node_ptr(&priv_header()), hint.pointed_node(), key, comp, commit_data));
+            (this->priv_header_ptr(), hint.pointed_node(), key, comp, commit_data));
       return std::pair<iterator, bool>(iterator(ret.first, this), ret.second);
    }
 
@@ -711,9 +732,9 @@ class rbtree_impl
       node_ptr to_insert(get_real_value_traits().to_node_ptr(value));
       if(safemode_or_autounlink)
          BOOST_INTRUSIVE_SAFE_HOOK_DEFAULT_ASSERT(node_algorithms::unique(to_insert));
-      this->priv_size_traits().increment();
       node_algorithms::insert_unique_commit
-               (node_ptr(&priv_header()), to_insert, commit_data);
+               (this->priv_header_ptr(), to_insert, commit_data);
+      this->priv_size_traits().increment();
       return iterator(to_insert, this);
    }
 
@@ -738,7 +759,7 @@ class rbtree_impl
          BOOST_INTRUSIVE_SAFE_HOOK_DEFAULT_ASSERT(node_algorithms::unique(to_insert));
       this->priv_size_traits().increment();
       return iterator(node_algorithms::insert_before
-         (node_ptr(&priv_header()), pos.pointed_node(), to_insert), this);
+         (this->priv_header_ptr(), pos.pointed_node(), to_insert), this);
    }
 
    //! <b>Requires</b>: value must be an lvalue, and it must be no less
@@ -761,7 +782,7 @@ class rbtree_impl
       if(safemode_or_autounlink)
          BOOST_INTRUSIVE_SAFE_HOOK_DEFAULT_ASSERT(node_algorithms::unique(to_insert));
       this->priv_size_traits().increment();
-      node_algorithms::push_back(node_ptr(&priv_header()), to_insert);
+      node_algorithms::push_back(this->priv_header_ptr(), to_insert);
    }
 
    //! <b>Requires</b>: value must be an lvalue, and it must be no greater
@@ -784,7 +805,7 @@ class rbtree_impl
       if(safemode_or_autounlink)
          BOOST_INTRUSIVE_SAFE_HOOK_DEFAULT_ASSERT(node_algorithms::unique(to_insert));
       this->priv_size_traits().increment();
-      node_algorithms::push_front(node_ptr(&priv_header()), to_insert);
+      node_algorithms::push_front(this->priv_header_ptr(), to_insert);
    }
 
    //! <b>Effects</b>: Erases the element pointed to by pos. 
@@ -802,7 +823,7 @@ class rbtree_impl
       node_ptr to_erase(i.pointed_node());
       if(safemode_or_autounlink)
          BOOST_INTRUSIVE_SAFE_HOOK_DEFAULT_ASSERT(!node_algorithms::unique(to_erase));
-      node_algorithms::erase(&priv_header(), to_erase);
+      node_algorithms::erase(this->priv_header_ptr(), to_erase);
       this->priv_size_traits().decrement();
       if(safemode_or_autounlink)
          node_algorithms::init(to_erase);
@@ -964,7 +985,7 @@ class rbtree_impl
          this->clear_and_dispose(detail::null_disposer());
       }
       else{
-         node_algorithms::init_header(&priv_header());
+         node_algorithms::init_header(this->priv_header_ptr());
          this->priv_size_traits().set_size(0);
       }
    }
@@ -981,9 +1002,9 @@ class rbtree_impl
    template<class Disposer>
    void clear_and_dispose(Disposer disposer)
    {
-      node_algorithms::clear_and_dispose(node_ptr(&priv_header())
+      node_algorithms::clear_and_dispose(this->priv_header_ptr()
          , detail::node_disposer<Disposer, rbtree_impl>(disposer, this));
-      node_algorithms::init_header(&priv_header());
+      node_algorithms::init_header(this->priv_header_ptr());
       this->priv_size_traits().set_size(0);
    }
 
@@ -1039,7 +1060,7 @@ class rbtree_impl
       detail::key_nodeptr_comp<KeyValueCompare, rbtree_impl>
          key_node_comp(comp, this);
       return iterator(node_algorithms::lower_bound
-         (const_node_ptr(&priv_header()), key, key_node_comp), this);
+         (this->priv_header_ptr(), key, key_node_comp), this);
    }
 
    //! <b>Effects</b>: Returns a const iterator to the first element whose
@@ -1054,7 +1075,7 @@ class rbtree_impl
       detail::key_nodeptr_comp<KeyValueCompare, rbtree_impl>
          key_node_comp(comp, this);
       return const_iterator(node_algorithms::lower_bound
-         (const_node_ptr(&priv_header()), key, key_node_comp), this);
+         (this->priv_header_ptr(), key, key_node_comp), this);
    }
 
    //! <b>Effects</b>: Returns an iterator to the first element whose
@@ -1079,7 +1100,7 @@ class rbtree_impl
       detail::key_nodeptr_comp<KeyValueCompare, rbtree_impl>
          key_node_comp(comp, this);
       return iterator(node_algorithms::upper_bound
-         (const_node_ptr(&priv_header()), key, key_node_comp), this);
+         (this->priv_header_ptr(), key, key_node_comp), this);
    }
 
    //! <b>Effects</b>: Returns an iterator to the first element whose
@@ -1104,7 +1125,7 @@ class rbtree_impl
       detail::key_nodeptr_comp<KeyValueCompare, rbtree_impl>
          key_node_comp(comp, this);
       return const_iterator(node_algorithms::upper_bound
-         (const_node_ptr(&priv_header()), key, key_node_comp), this);
+         (this->priv_header_ptr(), key, key_node_comp), this);
    }
 
    //! <b>Effects</b>: Finds an iterator to the first element whose key is 
@@ -1128,7 +1149,7 @@ class rbtree_impl
       detail::key_nodeptr_comp<KeyValueCompare, rbtree_impl>
          key_node_comp(comp, this);
       return iterator
-         (node_algorithms::find(const_node_ptr(&priv_header()), key, key_node_comp), this);
+         (node_algorithms::find(this->priv_header_ptr(), key, key_node_comp), this);
    }
 
    //! <b>Effects</b>: Finds a const_iterator to the first element whose key is 
@@ -1152,7 +1173,7 @@ class rbtree_impl
       detail::key_nodeptr_comp<KeyValueCompare, rbtree_impl>
          key_node_comp(comp, this);
       return const_iterator
-         (node_algorithms::find(const_node_ptr(&priv_header()), key, key_node_comp), this);
+         (node_algorithms::find(this->priv_header_ptr(), key, key_node_comp), this);
    }
 
    //! <b>Effects</b>: Finds a range containing all elements whose key is k or
@@ -1178,7 +1199,7 @@ class rbtree_impl
       detail::key_nodeptr_comp<KeyValueCompare, rbtree_impl>
          key_node_comp(comp, this);
       std::pair<node_ptr, node_ptr> ret
-         (node_algorithms::equal_range(const_node_ptr(&priv_header()), key, key_node_comp));
+         (node_algorithms::equal_range(this->priv_header_ptr(), key, key_node_comp));
       return std::pair<iterator, iterator>(iterator(ret.first, this), iterator(ret.second, this));
    }
 
@@ -1207,7 +1228,7 @@ class rbtree_impl
       detail::key_nodeptr_comp<KeyValueCompare, rbtree_impl>
          key_node_comp(comp, this);
       std::pair<node_ptr, node_ptr> ret
-         (node_algorithms::equal_range(const_node_ptr(&priv_header()), key, key_node_comp));
+         (node_algorithms::equal_range(this->priv_header_ptr(), key, key_node_comp));
       return std::pair<const_iterator, const_iterator>(const_iterator(ret.first, this), const_iterator(ret.second, this));
    }
 
@@ -1233,8 +1254,8 @@ class rbtree_impl
          detail::exception_disposer<rbtree_impl, Disposer>
             rollback(*this, disposer);
          node_algorithms::clone
-            (const_node_ptr(&src.priv_header())
-            ,node_ptr(&this->priv_header())
+            (const_node_ptr(src.priv_header_ptr())
+            ,node_ptr(this->priv_header_ptr())
             ,detail::node_cloner<Cloner, rbtree_impl>(cloner, this)
             ,detail::node_disposer<Disposer, rbtree_impl>(disposer, this));
          this->priv_size_traits().set_size(src.priv_size_traits().get_size());
@@ -1256,7 +1277,7 @@ class rbtree_impl
    pointer unlink_leftmost_without_rebalance()
    {
       node_ptr to_be_disposed(node_algorithms::unlink_leftmost_without_rebalance
-                           (node_ptr(&priv_header())));
+                           (this->priv_header_ptr()));
       if(!to_be_disposed)
          return 0;
       this->priv_size_traits().decrement();
@@ -1282,8 +1303,10 @@ class rbtree_impl
    void replace_node(iterator replace_this, reference with_this)
    {
       node_algorithms::replace_node( get_real_value_traits().to_node_ptr(*replace_this)
-                                   , node_ptr(&priv_header())
+                                   , this->priv_header_ptr()
                                    , get_real_value_traits().to_node_ptr(with_this));
+      if(safemode_or_autounlink)
+         node_algorithms::init(replace_this.pointed_node());
    }
 
    //! <b>Requires</b>: value must be an lvalue and shall be in a set of
@@ -1403,7 +1426,7 @@ class rbtree_impl
    static rbtree_impl &priv_container_from_end_iterator(const const_iterator &end_iterator)
    {
       header_plus_size *r = detail::parent_from_member<header_plus_size, node>
-         ( detail::boost_intrusive_get_pointer(end_iterator.pointed_node()), &header_plus_size::header_);
+         ( boost::intrusive::detail::to_raw_pointer(end_iterator.pointed_node()), &header_plus_size::header_);
       node_plus_pred_t *n = detail::parent_from_member
          <node_plus_pred_t, header_plus_size>(r, &node_plus_pred_t::header_plus_size_);
       data_t *d = detail::parent_from_member<data_t, node_plus_pred_t>(n, &data_t::node_plus_pred_);
@@ -1609,6 +1632,7 @@ class rbtree
       Options...
       #endif
       >::type   Base;
+   BOOST_MOVABLE_BUT_NOT_COPYABLE(rbtree)
 
    public:
    typedef typename Base::value_compare      value_compare;
@@ -1632,6 +1656,13 @@ class rbtree
       :  Base(unique, b, e, cmp, v_traits)
    {}
 
+   rbtree(BOOST_RV_REF(rbtree) x)
+      :  Base(::boost::move(static_cast<Base&>(x)))
+   {}
+
+   rbtree& operator=(BOOST_RV_REF(rbtree) x)
+   {  this->Base::operator=(::boost::move(static_cast<Base&>(x))); return *this;  }
+
    static rbtree &container_from_end_iterator(iterator end_iterator)
    {  return static_cast<rbtree &>(Base::container_from_end_iterator(end_iterator));   }
 
