@@ -4,7 +4,7 @@
     
     http://www.boost.org/
 
-    Copyright (c) 2001-2010 Hartmut Kaiser. Distributed under the Boost
+    Copyright (c) 2001-2011 Hartmut Kaiser. Distributed under the Boost
     Software License, Version 1.0. (See accompanying file
     LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 =============================================================================*/
@@ -100,7 +100,7 @@ public:
 // concept checks
 // the given iterator should be at least a forward iterator type
     BOOST_CLASS_REQUIRE(IteratorT, boost, ForwardIteratorConcept);
-    
+
 // public typedefs
     typedef typename LexIteratorT::token_type       token_type;
     typedef typename token_type::string_type        string_type;
@@ -136,6 +136,7 @@ public:
 #if BOOST_WAVE_SUPPORT_PRAGMA_ONCE != 0
       , current_filename(fname)
 #endif 
+      , current_relative_filename(fname)
       , macros(*this_())
       , language(language_support(
                       support_cpp 
@@ -164,7 +165,7 @@ public:
         std::string fname(filename);
         if (filename != "<Unknown>" && filename != "<stdin>") {
             using namespace boost::filesystem;
-            path fpath(complete(path(filename)));
+            path fpath(util::complete_path(path(filename)));
             fname = fpath.string();
         }
         return iterator_type(*this, first, last, position_type(fname.c_str())); 
@@ -176,7 +177,7 @@ public:
         std::string fname(filename);
         if (filename != "<Unknown>" && filename != "<stdin>") {
             using namespace boost::filesystem;
-            path fpath(complete(path(filename)));
+            path fpath(util::complete_path(path(filename)));
             fname = fpath.string();
         }
         return iterator_type(*this, first_, last_, position_type(fname.c_str())); 
@@ -228,16 +229,22 @@ public:
             has_params, is_predefined, pos, parameters, definition); 
     }
     template <typename StringT>
-    bool remove_macro_definition(StringT const &name, 
-            bool even_predefined = false)
+    bool remove_macro_definition(StringT const& undefname, bool even_predefined = false)
     { 
+        // strip leading and trailing whitespace
+        string_type name = util::to_string<string_type>(undefname);
+        typename string_type::size_type pos = name.find_first_not_of(" \t");
+        if (pos != string_type::npos) {
+            typename string_type::size_type endpos = name.find_last_not_of(" \t");
+            name = name.substr(pos, endpos-pos+1);
+        }
+
 #if BOOST_WAVE_SUPPORT_PRAGMA_ONCE != 0
         // ensure this gets removed from the list of include guards as well
         includes.remove_pragma_once_header(
             util::to_string<std::string>(name));
 #endif
-        return macros.remove_macro(util::to_string<string_type>(name), 
-            macros.get_main_pos(), even_predefined); 
+        return macros.remove_macro(name, macros.get_main_pos(), even_predefined); 
     }
     void reset_macro_definitions() 
         { macros.reset_macromap(); macros.init_predefined_macros(); }
@@ -284,6 +291,9 @@ public:
     }
     boost::wave::language_support get_language() const { return language; }
 
+    position_type &get_main_pos() { return macros.get_main_pos(); }
+    position_type const& get_main_pos() const { return macros.get_main_pos(); }
+
 // change and ask for maximal possible include nesting depth
     void set_max_include_nesting_depth(iter_size_type new_depth)
         { iter_ctxs.set_max_include_nesting_depth(new_depth); }
@@ -317,7 +327,7 @@ protected:
             std::string fname(filename);
             if (filename != "<Unknown>" && filename != "<stdin>") {
                 using namespace boost::filesystem;
-                path fpath(complete(path(filename)));
+                path fpath(util::complete_path(path(filename)));
                 fname = fpath.string();
                 includes.set_current_directory(fname.c_str());
             }
@@ -330,9 +340,6 @@ protected:
         { return macros.is_defined(begin, end); }
 
 // maintain include paths (helper functions)
-    bool find_include_file (std::string &s, std::string &d, bool is_system, 
-        char const *current_file) const
-    { return includes.find_include_file(s, d, is_system, current_file); }
     void set_current_directory(char const *path_) 
         { includes.set_current_directory(path_); }
 
@@ -357,8 +364,6 @@ protected:
     void push_iteration_context(position_type const &act_pos, iteration_ptr_type iter_ctx)
         { iter_ctxs.push(*this, act_pos, iter_ctx); }
 
-    position_type &get_main_pos() { return macros.get_main_pos(); }
-
 ///////////////////////////////////////////////////////////////////////////////
 //
 //  expand_tokensequence(): 
@@ -417,6 +422,15 @@ public:
     }
 #endif 
 
+    void set_current_relative_filename(char const *real_name)
+        { current_relative_filename = real_name; }
+    std::string const &get_current_relative_filename() const 
+        { return current_relative_filename; }
+
+    bool find_include_file (std::string &s, std::string &d, bool is_system, 
+        char const *current_file) const
+    { return includes.find_include_file(s, d, is_system, current_file); }
+
 #if BOOST_WAVE_SERIALIZATION != 0
 public:
     BOOST_STATIC_CONSTANT(unsigned int, version = 0x10);
@@ -502,6 +516,7 @@ private:
 #if BOOST_WAVE_SUPPORT_PRAGMA_ONCE != 0
     std::string current_filename;       // real name of current preprocessed file
 #endif 
+    std::string current_relative_filename;        // real relative name of current preprocessed file
 
     boost::wave::util::if_block_stack ifblocks;   // conditional compilation contexts
     boost::wave::util::include_paths includes;    // lists of include directories to search
