@@ -2,7 +2,7 @@
     Boost.Wave: A Standard compliant C++ preprocessor library
     http://www.boost.org/
 
-    Copyright (c) 2001-2010 Hartmut Kaiser. Distributed under the Boost
+    Copyright (c) 2001-2011 Hartmut Kaiser. Distributed under the Boost
     Software License, Version 1.0. (See accompanying file
     LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 =============================================================================*/
@@ -17,6 +17,7 @@
 #include <ostream>
 #include <string>
 #include <stack>
+#include <set>
 
 #include <boost/assert.hpp>
 #include <boost/config.hpp>
@@ -145,7 +146,8 @@ class trace_macro_expansion
     typedef boost::wave::context_policies::eat_whitespace<TokenT> base_type;
 
 public:
-    trace_macro_expansion(bool preserve_whitespace_, 
+    trace_macro_expansion(
+            bool preserve_whitespace_, bool preserve_bol_whitespace_,
             std::ofstream &output_, std::ostream &tracestrm_, 
             std::ostream &includestrm_, std::ostream &guardstrm_, 
             trace_flags flags_, bool enable_system_command_, 
@@ -155,12 +157,11 @@ public:
         level(0), flags(flags_), logging_flags(trace_nothing), 
         enable_system_command(enable_system_command_),
         preserve_whitespace(preserve_whitespace_),
+        preserve_bol_whitespace(preserve_bol_whitespace_),
         generate_output(generate_output_),
         default_outfile(default_outfile_),
         emit_relative_filenames(false)
     {
-        using namespace std;    // some systems have time in namespace std
-        time(&started_at);
     }
     ~trace_macro_expansion()
     {
@@ -184,6 +185,12 @@ public:
         return emit_relative_filenames;
     }
 
+    // add a macro name, which should not be expanded at all (left untouched)
+    void add_noexpandmacro(std::string const& name)
+    {
+        noexpandmacros.insert(name);
+    }
+
     ///////////////////////////////////////////////////////////////////////////
     //  
     //  The function 'expanding_function_like_macro' is called whenever a 
@@ -211,6 +218,11 @@ public:
     //  invocation (starting with the opening parenthesis and ending after the
     //  closing one).
     //
+    //  The return value defines whether the corresponding macro will be 
+    //  expanded (return false) or will be copied to the output (return true).
+    //  Note: the whole argument list is copied unchanged to the output as well
+    //        without any further processing.
+    //
     ///////////////////////////////////////////////////////////////////////////
 #if BOOST_WAVE_USE_DEPRECIATED_PREPROCESSING_HOOKS != 0
     // old signature
@@ -235,8 +247,15 @@ public:
         TokenT const &macrocall, std::vector<ContainerT> const &arguments,
         IteratorT const& seqstart, IteratorT const& seqend) 
     {
-        if (enabled_macro_counting())
-            count_invocation(macrodef.get_value().c_str());
+        if (enabled_macro_counting() || !noexpandmacros.empty()) {
+            std::string name (macrodef.get_value().c_str());
+
+            if (noexpandmacros.find(name.c_str()) != noexpandmacros.end())
+                return true;    // do not expand this macro
+
+            if (enabled_macro_counting())
+                count_invocation(name.c_str());
+        }
 
         if (!enabled_macro_tracing()) 
             return false;
@@ -354,8 +373,15 @@ public:
         TokenT const &macrodef, ContainerT const &definition, 
         TokenT const &macrocall)
     {
-        if (enabled_macro_counting())
-            count_invocation(macrodef.get_value().c_str());
+        if (enabled_macro_counting() || !noexpandmacros.empty()) {
+            std::string name (macrodef.get_value().c_str());
+
+            if (noexpandmacros.find(name.c_str()) != noexpandmacros.end())
+                return true;    // do not expand this macro
+
+            if (enabled_macro_counting())
+                count_invocation(name.c_str());
+        }
 
         if (!enabled_macro_tracing()) 
             return false;
@@ -741,7 +767,9 @@ public:
     bool may_skip_whitespace(ContextT const &ctx, TokenT &token, 
         bool &skipped_newline)
     {
-        return this->base_type::may_skip_whitespace(ctx, token, skipped_newline) ?
+        return this->base_type::may_skip_whitespace(
+                ctx, token, need_preserve_comments(ctx.get_language()), 
+                preserve_bol_whitespace, skipped_newline) ?
             !preserve_whitespace : false;
     }
 
@@ -781,7 +809,7 @@ protected:
         using namespace boost::wave;
         if (e.get_errorcode() != preprocess_exception::ill_formed_directive)
             return false;
-            
+
         // the error string is formatted as 'severity: error: directive'
         std::string error(e.description());
         std::string::size_type p = error.find_last_of(":");
@@ -841,24 +869,44 @@ protected:
     }
 
     ///////////////////////////////////////////////////////////////////////////
-    //  interpret the pragma wave option(preserve: [0|1|2|push|pop]) directive
+    //  interpret the pragma wave option(preserve: [0|1|2|3|push|pop]) directive
     template <typename ContextT>
     static bool 
     interpret_pragma_option_preserve_set(int mode, bool &preserve_whitespace, 
-        ContextT &ctx)
+        bool& preserve_bol_whitespace, ContextT &ctx)
     {
         switch(mode) {
-        case 0:   
+        // preserve no whitespace
+        case 0:
             preserve_whitespace = false;
+            preserve_bol_whitespace = false;
             ctx.set_language(
                 enable_preserve_comments(ctx.get_language(), false),
                 false);
             break;
 
-        case 2:
-            preserve_whitespace = true;
-              /* fall through */
+        // preserve BOL whitespace only
         case 1:
+            preserve_whitespace = false;
+            preserve_bol_whitespace = true;
+            ctx.set_language(
+                enable_preserve_comments(ctx.get_language(), false),
+                false);
+            break;
+
+        // preserve comments and BOL whitespace only
+        case 2:
+            preserve_whitespace = false;
+            preserve_bol_whitespace = true;
+            ctx.set_language(
+                enable_preserve_comments(ctx.get_language()),
+                false);
+            break;
+
+        // preserve all whitespace
+        case 3:
+            preserve_whitespace = true;
+            preserve_bol_whitespace = true;
             ctx.set_language(
                 enable_preserve_comments(ctx.get_language()),
                 false);
@@ -885,15 +933,18 @@ protected:
         if (T_IDENTIFIER == id) {
             if ((*it).get_value() == "push") {
             // push current preserve option onto the internal option stack
-                if (preserve_whitespace) {
-                    if (need_preserve_comments(ctx.get_language()))
-                        preserve_options.push(2);
+                if (need_preserve_comments(ctx.get_language())) {
+                    if (preserve_whitespace)
+                        preserve_options.push(3);
                     else
-                        preserve_options.push(1);
+                        preserve_options.push(2);
+                }
+                else if (preserve_bol_whitespace) {
+                    preserve_options.push(1);
                 }
                 else {
                     preserve_options.push(0);
-                }                    
+                }
                 return true;
             }
             else if ((*it).get_value() == "pop") {
@@ -903,10 +954,11 @@ protected:
                         pragma_mismatched_push_pop, "preserve", 
                         act_token.get_position());
                 }
-                
+
             // pop output preserve from the internal option stack
                 bool result = interpret_pragma_option_preserve_set(
-                    preserve_options.top(), preserve_whitespace, ctx);
+                    preserve_options.top(), preserve_whitespace, 
+                    preserve_bol_whitespace, ctx);
                 preserve_options.pop();
                 return result;
             }
@@ -918,10 +970,11 @@ protected:
 
         using namespace std;    // some platforms have atoi in namespace std
         return interpret_pragma_option_preserve_set(
-            atoi((*it).get_value().c_str()), preserve_whitespace, ctx);
+            atoi((*it).get_value().c_str()), preserve_whitespace, 
+            preserve_bol_whitespace, ctx);
     }
 
-    //  interpret the pragma wave option(line: [0|1|push|pop]) directive
+    //  interpret the pragma wave option(line: [0|1|2|push|pop]) directive
     template <typename ContextT, typename IteratorT>
     bool 
     interpret_pragma_option_line(ContextT &ctx, IteratorT &it,
@@ -970,7 +1023,7 @@ protected:
 
         using namespace std;    // some platforms have atoi in namespace std
         int emit_lines = atoi((*it).get_value().c_str());
-        if (0 == emit_lines || 1 == emit_lines) {
+        if (0 == emit_lines || 1 == emit_lines || 2 == emit_lines) {
             // set the new emit #line directive mode
             ctx.set_language(
                 enable_emit_line_directives(ctx.get_language(), emit_lines),
@@ -992,11 +1045,14 @@ protected:
         // ensure all directories for this file do exist
         fs::create_directories(boost::wave::util::branch_path(fpath));
 
-        // figure out, whether the file to open was last accessed by us
+        // figure out, whether the file has been written to by us, if yes, we
+        // append any output to this file, otherwise we overwrite it
         std::ios::openmode mode = std::ios::out;
-        if (fs::exists(fpath) && fs::last_write_time(fpath) >= started_at)
+        if (fs::exists(fpath) && written_by_us.find(fpath) != written_by_us.end())
             mode = (std::ios::openmode)(std::ios::out | std::ios::app);
 
+        written_by_us.insert(fpath);
+
         // close the current file
         if (outputstrm.is_open())
             outputstrm.close();
@@ -1045,7 +1101,7 @@ protected:
             string_type fname ((*it).get_value());
             fs::path fpath (boost::wave::util::create_path(
                 util::impl::unescape_lit(fname.substr(1, fname.size()-2)).c_str()));
-            fpath = fs::complete(fpath, ctx.get_current_directory());
+            fpath = boost::wave::util::complete_path(fpath, ctx.get_current_directory());
             result = interpret_pragma_option_output_open(fpath, ctx, act_token);
         }
         else if (T_IDENTIFIER == id) {
@@ -1058,8 +1114,8 @@ protected:
                 if (output_options.empty() && current_outfile.empty() &&
                     !default_outfile.empty() && default_outfile != "-")
                 {
-                    current_outfile = fs::complete(default_outfile, 
-                        ctx.get_current_directory());
+                    current_outfile = boost::wave::util::complete_path(
+                        default_outfile, ctx.get_current_directory());
                 }
 
             // push current output option onto the internal option stack
@@ -1114,11 +1170,72 @@ protected:
         return result;
     }
 
+    ///////////////////////////////////////////////////////////////////////////
+    // join all adjacent string tokens into the first one
+    template <typename StringT>
+    StringT unlit(StringT const& str)
+    {
+        return str.substr(1, str.size()-2);
+    }
+
+    template <typename StringT>
+    StringT merge_string_lits(StringT const& lhs, StringT const& rhs)
+    {
+        StringT result ("\"");
+
+        result += unlit(lhs);
+        result += unlit(rhs);
+        result += "\"";
+        return result;
+    }
+
+    template <typename ContextT, typename ContainerT>
+    void join_adjacent_string_tokens(ContextT &ctx, ContainerT const& values,
+        ContainerT& joined_values)
+    {
+        using namespace boost::wave;
+
+        typedef typename ContextT::token_type token_type;
+        typedef typename token_type::string_type string_type;
+        typedef typename ContainerT::const_iterator const_iterator;
+        typedef typename ContainerT::iterator iterator;
+
+        token_type* current = 0;
+
+        const_iterator end = values.end();
+        for (const_iterator it = values.begin(); it != end; ++it) {
+            token_id id(*it);
+
+            if (id == T_STRINGLIT) {
+                if (!current) {
+                    joined_values.push_back(*it);
+                    current = &joined_values.back();
+                }
+                else {
+                    current->set_value(merge_string_lits(
+                        current->get_value(), (*it).get_value()));
+                }
+            }
+            else if (current) {
+                typedef util::impl::next_token<const_iterator> next_token_type;
+                token_id next_id (next_token_type::peek(it, end, true));
+
+                if (next_id != T_STRINGLIT) {
+                    current = 0;
+                    joined_values.push_back(*it);
+                }
+            }
+            else {
+                joined_values.push_back(*it);
+            }
+        }
+    }
+
     ///////////////////////////////////////////////////////////////////////////
     //  interpret the pragma wave option() directives
     template <typename ContextT, typename ContainerT>
     bool 
-    interpret_pragma_option(ContextT &ctx, ContainerT const &values, 
+    interpret_pragma_option(ContextT &ctx, ContainerT const &cvalues, 
         typename ContextT::token_type const &act_token)
     {
         using namespace boost::wave;
@@ -1127,18 +1244,21 @@ protected:
         typedef typename token_type::string_type string_type;
         typedef typename ContainerT::const_iterator const_iterator;
 
+        ContainerT values;
+        join_adjacent_string_tokens(ctx, cvalues, values);
+
         const_iterator end = values.end();
         for (const_iterator it = values.begin(); it != end; /**/) {
         bool valid_option = false;
 
             token_type const &value = *it;
             if (value.get_value() == "preserve") {
-            // #pragma wave option(preserve: [0|1|2|push|pop])
+            // #pragma wave option(preserve: [0|1|2|3|push|pop])
                 valid_option = interpret_pragma_option_preserve(ctx, it, end, 
                     act_token);
             }
             else if (value.get_value() == "line") {
-            // #pragma wave option(line: [0|1|push|pop])
+            // #pragma wave option(line: [0|1|2|push|pop])
                 valid_option = interpret_pragma_option_line(ctx, it, end, 
                     act_token);
             }
@@ -1336,12 +1456,13 @@ private:
     trace_flags logging_flags;      // enabled by a #pragma
     bool enable_system_command;     // enable #pragma wave system() command
     bool preserve_whitespace;       // enable whitespace preservation
+    bool preserve_bol_whitespace;   // enable begin of line whitespace preservation
     bool& generate_output;          // allow generated tokens to be streamed to output
     std::string const& default_outfile;         // name of the output file given on command line
     boost::filesystem::path current_outfile;    // name of the current output file 
 
     stop_watch elapsed_time;        // trace timings
-    std::time_t started_at;         // time, this process was started at
+    std::set<boost::filesystem::path> written_by_us;    // all files we have written to
 
     typedef std::pair<bool, boost::filesystem::path> output_option_type;
     std::stack<output_option_type> output_options;  // output option stack
@@ -1350,6 +1471,8 @@ private:
 
     std::map<std::string, std::size_t> counts;    // macro invocation counts
     bool emit_relative_filenames;   // emit relative names in #line directives
+
+    std::set<std::string> noexpandmacros;   // list of macros not to expand
 };
 
 #undef BOOST_WAVE_GETSTRING
